#include <rclcpp/rclcpp.hpp>
#include <cv_bridge/cv_bridge.h>
#include <sensor_msgs/msg/image.hpp>
#include <opencv2/opencv.hpp>

/*** 名前空間を省略して利用できるように宣言 ***/
using sensor_msgs::msg::Image;

int main(int argc, char **argv)
{
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  auto node = rclcpp::Node::make_shared("image_publisher");
  /*** パラメータの初期化 ***/
  auto camera_path = node->declare_parameter<std::string>("camera_path", "/dev/video0");
  /*** USBカメラの初期化 ***/
  cv::VideoCapture camera(camera_path);
  /*** 初期化に失敗したらプロセス終了 ***/
  if(!camera.isOpened()){
    RCLCPP_ERROR(node->get_logger(), "Failed to open %s", camera_path.c_str());
    rclcpp::shutdown();
    return -1;
  }
  /*** パブリッシャの初期化 ***/
  auto pub = node->create_publisher<Image>("image", 10);
  /*** ループ周期の設定 ***/
  rclcpp::WallRate loop(30);
  /*** パブリッシュするメッセージの宣言 ***/
  Image ros_img;
  cv_bridge::CvImage cv_img;
  cv_img.encoding = "bgr8";
  /*** CTRL+Cで中断されるまで無限ループ ***/
  while(rclcpp::ok()){
    /*** USBカメラから画像を取得 ***/
    camera >> cv_img.image;
    cv_img.header.stamp = node->now();
    /*** OpenCVのMat型 -> ROS2のImage型への変換 ***/
    cv_img.toImageMsg(ros_img);
    /*** メッセージのパブリッシュ ***/
    pub->publish(ros_img);
    loop.sleep();
  }
  /*** プロセスの終了 ***/
  rclcpp::shutdown();
  return 0;
}