#include <rclcpp/rclcpp.hpp>
#include <cv_bridge/cv_bridge.h>
#include <sensor_msgs/msg/image.hpp>
#include <opencv2/opencv.hpp>

/*** 名前空間を省略して利用できるように宣言 ***/
using sensor_msgs::msg::Image;
/*** グローバル変数の宣言 ***/
rclcpp::Node::SharedPtr g_node = nullptr;

/*** Image型をサブスクライブするコールバック関数 ***/
void onImageSubscribed(const Image::SharedPtr msg)
{
  /*** ROS2のImage型 -> OpenCVのMat型への変換 ***/
  auto cv_img = cv_bridge::toCvShare(msg);
  /*** 画像の表示 ***/
  cv::imshow("Image", cv_img->image);
  cv::waitKey(1);
}

int main(int argc, char **argv)
{
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  g_node = rclcpp::Node::make_shared("image_subscriber");
  /*** サブスクライバの初期化 ***/
  auto sub = g_node->create_subscription<Image>("image", 10, onImageSubscribed);
  /*** コールバックの発生を待機 ***/
  rclcpp::spin(g_node);
  /*** プロセスの終了 ***/
  g_node = nullptr;
  rclcpp::shutdown();
  return 0;
}