#include <rclcpp/rclcpp.hpp>

/*** 名前空間を省略して利用できるように宣言 ***/
using namespace std::chrono_literals;

int main(int argc, char **argv)
{
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  auto node = rclcpp::Node::make_shared("parameter_client");
  /*** パラメータ クライアントの初期化 ***/
  auto client = std::make_shared<rclcpp::SyncParametersClient>(node, "parameter");
  /*** サービスが利用できるようになるまで待機 ***/
  while(!client->wait_for_service(1s) && rclcpp::ok()){
    RCLCPP_INFO(node->get_logger(), "Waiting for service...");
  }
  /*** パラメータの取得 ***/
  auto parameters = client->get_parameters({"int_val", "double_val", "str_val"});
  for(auto &param : parameters){
    std::cout << "Name : " << param.get_name() << std::endl
              << "Type : " << param.get_type_name() << std::endl
              << "Val : " << param.value_to_string() << std::endl << std::endl;
  }
  /*** プロセスの終了 ***/
  rclcpp::shutdown();
  return 0;
}