#include <rclcpp/rclcpp.hpp>
#include <std_srvs/srv/empty.hpp>

/*** 名前空間を省略して利用できるように宣言 ***/
using namespace std_srvs::srv;

int main(int argc, char **argv)
{
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  auto node = rclcpp::Node::make_shared("service_client");
  /*** サービス(クライアント)の初期化 ***/
  auto client = node->create_client<Empty>("say_hello");
  /*** サービスが有効になるまで待機***/
  while (!client->wait_for_service(std::chrono::seconds(1)) && rclcpp::ok()){
    RCLCPP_INFO(node->get_logger(), "waiting for service to appear...");
  }
  /*** サービスの呼び出し ***/
  auto request = std::make_shared<Empty::Request>();
  auto result = client->async_send_request(request);
  /*** サービスからの返答があるまで待機 ***/
  if(rclcpp::spin_until_future_complete(node, result) == rclcpp::FutureReturnCode::SUCCESS){
    RCLCPP_INFO(node->get_logger(), "The service was called successfully");
  } else {
    RCLCPP_ERROR(node->get_logger(), "Failed to call the service");
  }
  /*** プロセスの終了 ***/
  rclcpp::shutdown();
  return 0;
}