#include <rclcpp/rclcpp.hpp>
#include <tutorial_interfaces/srv/add_two_ints.hpp>

/*** 名前空間を省略して利用できるように宣言 ***/
using namespace tutorial_interfaces::srv;

int main(int argc, char **argv)
{
  /*** コマンドライン引数の確認 ***/
  if(argc < 3){
    std::cout << "Usage is... " << argv[0] << " [val1] [val2]" << std::endl;
    return -1;
  }
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  auto node = rclcpp::Node::make_shared("service_client");
  /*** サービス(クライアント)の初期化 ***/
  auto client = node->create_client<AddTwoInts>("add_two_ints");
  /*** サービスが有効になるまで待機 ***/
  while (!client->wait_for_service(std::chrono::seconds(1)) && rclcpp::ok()){
    RCLCPP_INFO(node->get_logger(), "waiting for service to appear...");
  }
  /*** リクエストに数値を代入 ***/
  auto request = std::make_shared<AddTwoInts::Request>();
  request->val1 = std::stoi(argv[1]);
  request->val2 = std::stoi(argv[2]);
  /*** サービスの呼び出し ***/
  auto result = client->async_send_request(request);
  /*** サービスからの返答があるまで待機 ***/
  if(rclcpp::spin_until_future_complete(node, result) == rclcpp::FutureReturnCode::SUCCESS){
    RCLCPP_INFO(node->get_logger(), "%d + %d = %d", request->val1, request->val2, result.get()->sum);
  } else {
    RCLCPP_ERROR(node->get_logger(), "Failed to call the service");
  }
  /*** プロセスの終了 ***/
  rclcpp::shutdown();
  return 0;
}