#include <rclcpp/rclcpp.hpp>
#include <tutorial_interfaces/srv/add_two_ints.hpp>

/*** 名前空間を省略して利用できるように宣言 ***/
using namespace tutorial_interfaces::srv;
/*** グローバル変数の宣言 ***/
rclcpp::Node::SharedPtr g_node = nullptr;

/*** サービスのコールバック関数 ***/
void serviceAddTwoInts(const std::shared_ptr<rmw_request_id_t> header,
                       const std::shared_ptr<AddTwoInts::Request> req,
                       const std::shared_ptr<AddTwoInts::Response> res)
{
  res->sum = req->val1 + req->val2;
  RCLCPP_INFO(g_node->get_logger(), "Service was called");
}

int main(int argc, char **argv)
{
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  g_node = rclcpp::Node::make_shared("service_server");
  /*** サービス(サーバ)の初期化 ***/
  auto server = g_node->create_service<AddTwoInts>("add_two_ints", serviceAddTwoInts);
  /*** コールバックの発生を待機 ***/
  rclcpp::spin(g_node);
  /*** プロセスの終了 ***/
  g_node = nullptr;
  rclcpp::shutdown();
  return 0;
}