#include <rclcpp/rclcpp.hpp>
#include <tutorial_interfaces/msg/int32_stamped.hpp>
#include <message_filters/subscriber.h>
#include <message_filters/sync_policies/approximate_time.h>

/*** 名前空間を省略して利用できるように宣言 ***/
using tutorial_interfaces::msg::Int32Stamped;
using message_filters::sync_policies::ApproximateTime;
using message_filters::Synchronizer;
using std::placeholders::_1;
using std::placeholders::_2;
/*** グローバル変数の宣言 ***/
rclcpp::Node::SharedPtr g_node = nullptr;

/*** 2つのInt32Stamped型をサブスクライブするコールバック関数 ***/
void onNumbersSubscribed(Int32Stamped::ConstSharedPtr msg1, Int32Stamped::ConstSharedPtr msg2)
{
  RCLCPP_INFO(g_node->get_logger(), "I heard : %d and %d", msg1->data, msg2->data);
}

int main(int argc, char **argv)
{
  /*** プロセスの初期化 ***/
  rclcpp::init(argc, argv);
  /*** ノードの初期化 ***/
  g_node = rclcpp::Node::make_shared("subscriber");
  /*** サブスクライバの初期化 ***/
  typedef ApproximateTime<Int32Stamped, Int32Stamped> SyncPolicy;
  std::shared_ptr<message_filters::Subscriber<Int32Stamped>> sub1, sub2;
  std::shared_ptr<Synchronizer<SyncPolicy>> sync_msgs_;
  sub1.reset(new message_filters::Subscriber<Int32Stamped>(g_node, "number1"));
  sub2.reset(new message_filters::Subscriber<Int32Stamped>(g_node, "number2"));
  sync_msgs_.reset(new Synchronizer<SyncPolicy>(SyncPolicy(10), *sub1, *sub2));
  sync_msgs_->registerCallback(std::bind(onNumbersSubscribed, _1, _2));
  /*** コールバックの発生を待機 ***/
  rclcpp::spin(g_node);
  /*** プロセスの終了 ***/
  g_node = nullptr;
  rclcpp::shutdown();
  return 0;
}